package java.beans;

import org.xml.sax.helpers.DefaultHandler;
import java.security.PrivilegedAction;
import java.io.IOException;
import java.io.Closeable;
import java.security.AccessController;
import java.io.InputStream;
import org.xml.sax.InputSource;
import com.sun.beans.decoder.DocumentHandler;
import java.security.AccessControlContext;

public class XMLDecoder implements AutoCloseable
{
    private final AccessControlContext acc;
    private final DocumentHandler handler;
    private final InputSource input;
    private Object owner;
    private Object[] array;
    private int index;
    
    public XMLDecoder(InputStream inputStream) {
        this(inputStream, null);
    }
    
    public XMLDecoder(InputStream inputStream, Object o) {
        this(inputStream, o, null);
    }
    
    public XMLDecoder(InputStream inputStream, Object o, ExceptionListener exceptionListener) {
        this(inputStream, o, exceptionListener, null);
    }
    
    public XMLDecoder(InputStream byteStream, Object o, ExceptionListener exceptionListener, ClassLoader classLoader) {
        this(new InputSource(byteStream), o, exceptionListener, classLoader);
    }
    
    public XMLDecoder(InputSource inputSource) {
        this(inputSource, null, null, null);
    }
    
    private XMLDecoder(InputSource input, Object owner, ExceptionListener exceptionListener, ClassLoader classLoader) {
        this.acc = AccessController.getContext();
        this.handler = new DocumentHandler();
        this.input = input;
        this.owner = owner;
        this.setExceptionListener(exceptionListener);
        this.handler.setClassLoader(classLoader);
        this.handler.setOwner(this);
    }
    
    @Override
    public void close() {
        if (this.parsingComplete()) {
            this.close(this.input.getCharacterStream());
            this.close(this.input.getByteStream());
        }
    }
    
    private void close(Closeable closeable) {
        if (closeable != null) {
            try {
                closeable.close();
            }
            catch (IOException ex) {
                this.getExceptionListener().exceptionThrown(ex);
            }
        }
    }
    
    private boolean parsingComplete() {
        if (this.input == null) {
            return false;
        }
        if (this.array == null) {
            if (this.acc == null && null != System.getSecurityManager()) {
                throw new SecurityException("AccessControlContext is not set");
            }
            AccessController.doPrivileged(new PrivilegedAction<Void>() {
                @Override
                public Void run() {
                    XMLDecoder.this.handler.parse(XMLDecoder.this.input);
                    return null;
                }
            }, this.acc);
            this.array = this.handler.getObjects();
        }
        return true;
    }
    
    public void setExceptionListener(ExceptionListener defaultExceptionListener) {
        if (defaultExceptionListener == null) {
            defaultExceptionListener = Statement.defaultExceptionListener;
        }
        this.handler.setExceptionListener(defaultExceptionListener);
    }
    
    public ExceptionListener getExceptionListener() {
        return this.handler.getExceptionListener();
    }
    
    public Object readObject() {
        return this.parsingComplete() ? this.array[this.index++] : null;
    }
    
    public void setOwner(Object owner) {
        this.owner = owner;
    }
    
    public Object getOwner() {
        return this.owner;
    }
    
    public static DefaultHandler createHandler(Object owner, ExceptionListener exceptionListener, ClassLoader classLoader) {
        DocumentHandler documentHandler = new DocumentHandler();
        documentHandler.setOwner(owner);
        documentHandler.setExceptionListener(exceptionListener);
        documentHandler.setClassLoader(classLoader);
        return documentHandler;
    }
}
